@@ -20,47 +20,12 @@ from torch.optim.optimizer import Optimizer
 from torch.utils.data import DataLoader
 
 from pytorch_lightning.utilities import move_data_to_device, rank_zero_warn
+from pytorch_lightning.utilities.types import STEP_OUTPUT
 
 
 class ModelHooks:
     """Hooks to be used in LightningModule."""
 
-    def setup(self, stage: str) -> None:
-        """
-        Called at the beginning of fit and test.
-        This is a good hook when you need to build models dynamically or adjust something about them.
-        This hook is called on every process when using DDP.
-
-        Args:
-            stage: either 'fit' or 'test'
-
-        Example::
-
-            class LitModel(...):
-                def __init__(self):
-                    self.l1 = None
-
-                def prepare_data(self):
-                    download_data()
-                    tokenize()
-
-                    # don't do this
-                    self.something = else
-
-                def setup(stage):
-                    data = Load_data(...)
-                    self.l1 = nn.Linear(28, data.num_classes)
-
-        """
-
-    def teardown(self, stage: str) -> None:
-        """
-        Called at the end of fit and test.
-
-        Args:
-            stage: either 'fit' or 'test'
-        """
-
     def on_fit_start(self) -> None:
         """
         Called at the very beginning of fit.
@@ -77,25 +42,41 @@ class ModelHooks:
         """
         Called at the beginning of training after sanity check.
         """
-        # do something at the start of training
 
     def on_train_end(self) -> None:
         """
         Called at the end of training before logger experiment is closed.
         """
-        # do something at the end of training
 
     def on_validation_start(self) -> None:
         """
         Called at the beginning of validation.
         """
-        # do something at the start of validation
 
     def on_validation_end(self) -> None:
         """
         Called at the end of validation.
         """
-        # do something at the end of validation
+
+    def on_test_start(self) -> None:
+        """
+        Called at the beginning of testing.
+        """
+
+    def on_test_end(self) -> None:
+        """
+        Called at the end of testing.
+        """
+
+    def on_predict_start(self) -> None:
+        """
+        Called at the beginning of predicting.
+        """
+
+    def on_predict_end(self) -> None:
+        """
+        Called at the end of predicting.
+        """
 
     def on_pretrain_routine_start(self) -> None:
         """
@@ -107,7 +88,6 @@ class ModelHooks:
         - training_start
 
         """
-        # do something at the start of the pretrain routine
 
     def on_pretrain_routine_end(self) -> None:
         """
@@ -119,7 +99,6 @@ class ModelHooks:
         - training_start
 
         """
-        # do something at the end of the pretrain routine
 
     def on_train_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
         """
@@ -132,9 +111,8 @@ class ModelHooks:
             batch_idx: the index of the batch
             dataloader_idx: the index of the dataloader
         """
-        # do something when the batch starts
 
-    def on_train_batch_end(self, outputs: Any, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
+    def on_train_batch_end(self, outputs: STEP_OUTPUT, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
         """
         Called in the training loop after the batch.
 
@@ -144,7 +122,73 @@ class ModelHooks:
             batch_idx: the index of the batch
             dataloader_idx: the index of the dataloader
         """
-        # do something when the batch ends
+
+    def on_validation_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
+        """
+        Called in the validation loop before anything happens for that batch.
+
+        Args:
+            batch: The batched data as it is returned by the validation DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
+
+    def on_validation_batch_end(
+        self, outputs: Optional[STEP_OUTPUT], batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        """
+        Called in the validation loop after the batch.
+
+        Args:
+            outputs: The outputs of validation_step_end(validation_step(x))
+            batch: The batched data as it is returned by the validation DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
+
+    def on_test_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
+        """
+        Called in the test loop before anything happens for that batch.
+
+        Args:
+            batch: The batched data as it is returned by the test DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
+
+    def on_test_batch_end(
+        self, outputs: Optional[STEP_OUTPUT], batch: Any, batch_idx: int, dataloader_idx: int
+    ) -> None:
+        """
+        Called in the test loop after the batch.
+
+        Args:
+            outputs: The outputs of test_step_end(test_step(x))
+            batch: The batched data as it is returned by the test DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
+
+    def on_predict_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
+        """
+        Called in the predict loop before anything happens for that batch.
+
+        Args:
+            batch: The batched data as it is returned by the test DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
+
+    def on_predict_batch_end(self, outputs: Optional[Any], batch: Any, batch_idx: int, dataloader_idx: int) -> None:
+        """
+        Called in the predict loop after the batch.
+
+        Args:
+            outputs: The outputs of predict_step_end(test_step(x))
+            batch: The batched data as it is returned by the test DataLoader.
+            batch_idx: the index of the batch
+            dataloader_idx: the index of the dataloader
+        """
 
     def on_validation_model_eval(self) -> None:
         """
@@ -158,52 +202,6 @@ class ModelHooks:
         """
         self.trainer.model.train()
 
-    def on_validation_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
-        """
-        Called in the validation loop before anything happens for that batch.
-
-        Args:
-            batch: The batched data as it is returned by the validation DataLoader.
-            batch_idx: the index of the batch
-            dataloader_idx: the index of the dataloader
-        """
-        # do something when the batch starts
-
-    def on_validation_batch_end(self, outputs: Any, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
-        """
-        Called in the validation loop after the batch.
-
-        Args:
-            outputs: The outputs of validation_step_end(validation_step(x))
-            batch: The batched data as it is returned by the validation DataLoader.
-            batch_idx: the index of the batch
-            dataloader_idx: the index of the dataloader
-        """
-        # do something when the batch ends
-
-    def on_test_batch_start(self, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
-        """
-        Called in the test loop before anything happens for that batch.
-
-        Args:
-            batch: The batched data as it is returned by the test DataLoader.
-            batch_idx: the index of the batch
-            dataloader_idx: the index of the dataloader
-        """
-        # do something when the batch starts
-
-    def on_test_batch_end(self, outputs: Any, batch: Any, batch_idx: int, dataloader_idx: int) -> None:
-        """
-        Called in the test loop after the batch.
-
-        Args:
-            outputs: The outputs of test_step_end(test_step(x))
-            batch: The batched data as it is returned by the test DataLoader.
-            batch_idx: the index of the batch
-            dataloader_idx: the index of the dataloader
-        """
-        # do something when the batch ends
-
     def on_test_model_train(self) -> None:
         """
         Sets the model to train during the test loop
@@ -226,65 +224,60 @@ class ModelHooks:
         """
         Called when either of train/val/test epoch begins.
         """
-        # do something when the epoch starts
 
     def on_epoch_end(self) -> None:
         """
         Called when either of train/val/test epoch ends.
         """
-        # do something when the epoch ends
 
     def on_train_epoch_start(self) -> None:
         """
         Called in the training loop at the very beginning of the epoch.
         """
-        # do something when the epoch starts
 
-    def on_train_epoch_end(self, outputs) -> None:
+    def on_train_epoch_end(self, unused: Optional = None) -> None:
         """
         Called in the training loop at the very end of the epoch.
+
+        To access all batch outputs at the end of the epoch, either:
+
+        1. Implement `training_epoch_end` in the LightningModule OR
+        2. Cache data across steps on the attribute(s) of the `LightningModule` and access them in this hook
         """
-        # do something when the epoch ends
 
     def on_validation_epoch_start(self) -> None:
         """
         Called in the validation loop at the very beginning of the epoch.
         """
-        # do something when the epoch starts
 
     def on_validation_epoch_end(self) -> None:
         """
         Called in the validation loop at the very end of the epoch.
         """
-        # do something when the epoch ends
 
     def on_test_epoch_start(self) -> None:
         """
         Called in the test loop at the very beginning of the epoch.
         """
-        # do something when the epoch starts
 
     def on_test_epoch_end(self) -> None:
         """
         Called in the test loop at the very end of the epoch.
         """
-        # do something when the epoch ends
 
-    def on_test_start(self) -> None:
+    def on_predict_epoch_start(self) -> None:
         """
-        Called at the beginning of testing.
+        Called at the beginning of predicting.
         """
-        # do something at the start of testing
 
-    def on_test_end(self) -> None:
+    def on_predict_epoch_end(self, results: List[Any]) -> None:
         """
-        Called at the end of testing.
+        Called at the end of predicting.
         """
-        # do something at the end of testing
 
     def on_before_zero_grad(self, optimizer: Optimizer) -> None:
         """
-        Called after optimizer.step() and before optimizer.zero_grad().
+        Called after ``training_step()`` and before ``optimizer.zero_grad()``.
 
         Called in the training loop after taking an optimizer step and before zeroing grads.
         Good place to inspect weight information with weights updated.
@@ -292,14 +285,16 @@ class ModelHooks:
         This is where it is called::
 
             for optimizer in optimizers:
-                optimizer.step()
+                out = training_step(...)
+
                 model.on_before_zero_grad(optimizer) # < ---- called here
                 optimizer.zero_grad()
 
+                backward()
+
         Args:
             optimizer: The optimizer for which grads should be zeroed.
         """
-        # do something with the optimizer or inspect it.
 
     def on_after_backward(self) -> None:
         """
@@ -334,6 +329,20 @@ class ModelHooks:
 
         """
 
+    def configure_sharded_model(self) -> None:
+        """
+        Hook to create modules in a distributed aware context. This is useful for when using sharded plugins,
+        where we'd like to shard the model instantly, which is useful for extremely large models
+        which can save memory and initialization time.
+
+        The accelerator manages whether to call this hook at every given stage.
+        For sharded plugins where model parallelism is required, the hook is usually on called once
+        to initialize the sharded parameters, and not called again in the same process.
+
+        By default for accelerators/plugins that do not use model sharding techniques,
+        this hook is called during each fit/val/test/predict stages.
+        """
+
 
 class DataHooks:
     """Hooks to be used for data related stuff."""
@@ -383,7 +392,43 @@ class DataHooks:
             model.test_dataloader()
         """
 
-    def train_dataloader(self) -> Any:
+    def setup(self, stage: Optional[str] = None) -> None:
+        """
+        Called at the beginning of fit (train + validate), validate, test, and predict.
+        This is a good hook when you need to build models dynamically or adjust something about them.
+        This hook is called on every process when using DDP.
+
+        Args:
+            stage: either ``'fit'``, ``'validate'``, ``'test'``, or ``'predict'``
+
+        Example::
+
+            class LitModel(...):
+                def __init__(self):
+                    self.l1 = None
+
+                def prepare_data(self):
+                    download_data()
+                    tokenize()
+
+                    # don't do this
+                    self.something = else
+
+                def setup(stage):
+                    data = Load_data(...)
+                    self.l1 = nn.Linear(28, data.num_classes)
+
+        """
+
+    def teardown(self, stage: Optional[str] = None) -> None:
+        """
+        Called at the end of fit (train + validate), validate, test, predict, or tune.
+
+        Args:
+            stage: either ``'fit'``, ``'validate'``, ``'test'``, or ``'predict'``
+        """
+
+    def train_dataloader(self) -> Union[DataLoader, List[DataLoader], Dict[str, DataLoader]]:
         """
         Implement one or more PyTorch DataLoaders for training.
 
@@ -594,6 +639,18 @@ class DataHooks:
             will have an argument ``dataloader_idx`` which matches the order here.
         """
 
+    def on_train_dataloader(self) -> None:
+        """Called before requesting the train dataloader."""
+
+    def on_val_dataloader(self) -> None:
+        """Called before requesting the val dataloader."""
+
+    def on_test_dataloader(self) -> None:
+        """Called before requesting the test dataloader."""
+
+    def on_predict_dataloader(self) -> None:
+        """Called before requesting the predict dataloader."""
+
     def transfer_batch_to_device(self, batch: Any, device: Optional[torch.device] = None) -> Any:
         """
         Override this hook if your :class:`~torch.utils.data.DataLoader` returns tensors
@@ -646,7 +703,7 @@ class DataHooks:
         device = device or self.device
         return move_data_to_device(batch, device)
 
-    def on_before_batch_transfer(self, batch, dataloader_idx):
+    def on_before_batch_transfer(self, batch: Any, dataloader_idx: int) -> Any:
         """
         Override to alter or apply batch augmentations to your batch before it is transferred to the device.
 
@@ -679,7 +736,7 @@ class DataHooks:
         """
         return batch
 
-    def on_after_batch_transfer(self, batch, dataloader_idx):
+    def on_after_batch_transfer(self, batch: Any, dataloader_idx: int) -> Any:
         """
         Override to alter or apply batch augmentations to your batch after it is transferred to the device.
 
