@@ -22,6 +22,7 @@ from torch.nn.parallel import DistributedDataParallel
 from pytorch_lightning.core.lightning import LightningModule
 from pytorch_lightning.overrides.base import _LightningModuleWrapperBase
 from pytorch_lightning.overrides.distributed import LightningDistributedModule
+from pytorch_lightning.utilities import rank_zero_warn
 from pytorch_lightning.utilities.apply_func import apply_to_collection
 
 
@@ -71,6 +72,8 @@ class LightningParallelModule(_LightningModuleWrapperBase):
         super().__init__(pl_module)
 
     def forward(self, *inputs, **kwargs):
+        self.update_replica_device_attributes(inputs)
+        # forward call will redirect to training_step, validation_step, etc.
         output = super().forward(*inputs, **kwargs)
 
         def output_transform(data: Any):
@@ -85,6 +88,37 @@ class LightningParallelModule(_LightningModuleWrapperBase):
         )
         return output
 
+    def update_replica_device_attributes(self, inputs: Any) -> None:
+        """
+        Updates the device information of LightningModule by reading the device from the inputs.
+        In :class:`~torch.nn.data_parallel.DataParallel` changes to the state during the `forward` pass
+        are lost when the replicas get discarded. The only way to know the current device is from the
+        inputs passed into the model.
+
+        Args:
+            inputs: A collection of inputs (typically a tuple). If the inputs don't contain tensors,
+                a warning is shown that accessing ``self.device`` will not return the correct device.
+        """
+        replica_device = None
+
+        def find_tensor_with_device(tensor: torch.Tensor) -> torch.Tensor:
+            nonlocal replica_device
+            if replica_device is None and tensor.device != torch.device("cpu"):
+                replica_device = tensor.device
+            return tensor
+
+        apply_to_collection(inputs, dtype=torch.Tensor, function=find_tensor_with_device)
+
+        if replica_device is not None:
+            # by calling .to() we force the update to the self.device property
+            self.module.to(device=replica_device)
+        else:
+            rank_zero_warn(
+                "Could not determine on which device the inputs are."
+                " When using DataParallel (accelerator='dp'), be aware that in case you are using self.device"
+                " in your code, it will reference only the root device."
+            )
+
 
 def python_scalar_to_tensor(data: Any, device: torch.device = torch.device("cpu")) -> Any:
     """ Converts a Python scalar number to a torch tensor and places it on the given device. """
