@@ -12,48 +12,56 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 import math
-from typing import Any, Callable, Generator, Sequence, Tuple, Union
+from typing import Any, Callable, List, Optional, Tuple, Union
 
 import torch
+from torch import Tensor
 from torch.nn import Module
 from torch.optim import Optimizer
 
-from pytorch_lightning.core import LightningModule
+import pytorch_lightning as pl
 from pytorch_lightning.plugins.base_plugin import Plugin
+from pytorch_lightning.utilities import GradClipAlgorithmType
+from pytorch_lightning.utilities.types import _PARAMETERS
 
 
 class PrecisionPlugin(Plugin):
-    """ Plugin handling the precision-specific parts of the training.
+    """
+    Base class for all plugins handling the precision-specific parts of the training.
     The static classattributes EPSILON and precision must be overwritten in child-classes and their
     default values reflect fp32 training.
     """
-    EPSILON = 1e-6
-    precision = 32
+    EPSILON: float = 1e-6
+    precision: Union[str, int] = 32
 
-    def master_params(self, optimizer: Optimizer) -> Generator[torch.Tensor, None, None]:
-        """The master params of the model. Returns the plain model params here.
+    def master_params(self, optimizer: Optimizer) -> _PARAMETERS:
+        """
+        The master params of the model. Returns the plain model params here.
         Maybe different in other precision plugins.
-
         """
         for group in optimizer.param_groups:
             for p in group["params"]:
                 yield p
 
-    def connect(self, model: Module, optimizers: Sequence,
-                lr_schedulers: Sequence) -> Tuple[Module, Sequence, Sequence]:
+    def connect(
+        self,
+        model: Module,
+        optimizers: List[Optimizer],
+        lr_schedulers: List[Any],
+    ) -> Tuple[Module, List[Optimizer], List[Any]]:
         """Connects this plugin to the accelerator and the training process"""
         return model, optimizers, lr_schedulers
 
     def backward(
         self,
-        model: LightningModule,
-        closure_loss: torch.Tensor,
+        model: 'pl.LightningModule',
+        closure_loss: Tensor,
         optimizer: Optimizer,
         opt_idx: int,
         should_accumulate: bool,
         *args: Any,
         **kwargs: Any,
-    ) -> torch.Tensor:
+    ) -> Tensor:
         """performs the actual backpropagation
 
         Args:
@@ -78,7 +86,12 @@ class PrecisionPlugin(Plugin):
         return closure_loss
 
     def pre_optimizer_step(
-        self, pl_module: LightningModule, optimizer: Optimizer, optimizer_idx: int, closure: Callable, **kwargs
+        self,
+        pl_module: 'pl.LightningModule',
+        optimizer: Optimizer,
+        optimizer_idx: int,
+        lambda_closure: Callable,
+        **kwargs: Any,
     ) -> bool:
         """Hook to do something before each optimizer step."""
         return True
@@ -86,24 +99,40 @@ class PrecisionPlugin(Plugin):
     def post_optimizer_step(self, optimizer: Optimizer, optimizer_idx: int) -> None:
         """Hook to do something after each optimizer step."""
 
-    def clip_gradients(self, optimizer: Optimizer, clip_val: Union[int, float], norm_type: float = float(2.0)) -> None:
-        """Clips the gradients to a specific value"""
+    def clip_gradients(
+        self,
+        optimizer: Optimizer,
+        clip_val: Union[int, float],
+        gradient_clip_algorithm: GradClipAlgorithmType = GradClipAlgorithmType.NORM,
+        model: Optional[Module] = None
+    ) -> None:
+        """Clips the gradients"""
         if clip_val is None:
             return
 
-        grad_clip_val = float(clip_val)
-
-        if grad_clip_val <= 0:
+        clip_val = float(clip_val)
+        if clip_val <= 0:
             return
 
-        parameters = list(self.master_params(optimizer))
+        if gradient_clip_algorithm == GradClipAlgorithmType.VALUE:
+            self.clip_grad_by_value(optimizer, clip_val)
+        elif gradient_clip_algorithm == GradClipAlgorithmType.NORM:
+            # TODO: there should be a mechanism to set `norm_type`
+            self.clip_grad_by_norm(optimizer, clip_val, eps=self.EPSILON)
 
-        max_norm = grad_clip_val
+    def clip_grad_by_value(self, optimizer: Optimizer, clip_val: Union[int, float]) -> None:
+        """Clip gradients by value"""
+        parameters = self.master_params(optimizer)
+        torch.nn.utils.clip_grad_value_(parameters, clip_value=clip_val)
 
-        if isinstance(parameters, torch.Tensor):
-            parameters = [parameters]
+    def clip_grad_by_norm(
+        self, optimizer: Optimizer, clip_val: Union[int, float], norm_type: float = 2.0, eps: float = 1e-6
+    ) -> None:
+        """Clip gradients by norm"""
+        parameters = self.master_params(optimizer)
+
+        # TODO: replace this with torch.nn.clip_grad_norm_
         parameters = list(filter(lambda p: p.grad is not None, parameters))
-
         device = parameters[0].device
 
         if norm_type == math.inf:
@@ -114,9 +143,7 @@ class PrecisionPlugin(Plugin):
                 torch.norm(p.grad.data.to(device), norm_type, out=out[i])
             total_norm = torch.norm(out, norm_type)
 
-        eps = self.EPSILON
-
-        clip_coef = torch.tensor(max_norm, device=device) / (total_norm + eps)
+        clip_coef = torch.tensor(clip_val, device=device) / (total_norm + eps)
         clip_coef = torch.min(clip_coef, torch.ones_like(clip_coef))
         for p in parameters:
             p.grad.data.mul_(clip_coef.to(p.grad.data.device))
