@@ -11,9 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-from typing import cast, Union
-
-from torch.optim import Optimizer
+from typing import Union
 
 from pytorch_lightning.plugins.precision.native_amp import NativeMixedPrecisionPlugin
 from pytorch_lightning.utilities import _FAIRSCALE_AVAILABLE, _NATIVE_AMP_AVAILABLE
@@ -24,16 +22,13 @@ if _NATIVE_AMP_AVAILABLE and _FAIRSCALE_AVAILABLE:
 
 
 class ShardedNativeMixedPrecisionPlugin(NativeMixedPrecisionPlugin):
-    """Mixed Precision for Sharded Training
-    """
+    """Mixed Precision for Sharded Training"""
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
         self.scaler = ShardedGradScaler()
 
-    def clip_gradients(self, optimizer: 'Optimizer', clip_val: Union[int, float], norm_type: float = 2.0) -> None:
-        if clip_val <= 0:
-            return
-
-        optimizer = cast(OSS, optimizer)
+    def clip_grad_by_norm(
+        self, optimizer: 'OSS', clip_val: Union[int, float], norm_type: float = 2.0, eps: float = 1e-6
+    ) -> None:
         optimizer.clip_grad_norm(clip_val, norm_type=norm_type)
